From 810d41b1b7f67acab90cb00cc8191a298a49ed70 Mon Sep 17 00:00:00 2001
From: Angus Gratton <gus@projectgus.com>
Date: Tue, 1 Jul 2014 20:04:07 +1000
Subject: [PATCH 5/5] Add simple sum routine as suggested by Andreas

Change-Id: I691c3e332754b0d3cae70a1d2ee9cb77147f6714
---
 contrib/loaders/flash/cortex-m0.S |  4 ++++
 src/flash/nor/nrf51.c             | 21 ++++++++++++++-------
 2 files changed, 18 insertions(+), 7 deletions(-)

diff --git a/contrib/loaders/flash/cortex-m0.S b/contrib/loaders/flash/cortex-m0.S
index 6a6c84d..173b4d0 100644
--- a/contrib/loaders/flash/cortex-m0.S
+++ b/contrib/loaders/flash/cortex-m0.S
@@ -47,8 +47,10 @@
 	 * r4 - rp
 	 * r5 - wp, tmp
 	 * r6 - tmp
+         * r7 - in-memory sum
 	 */
 
+        movs     r7, #0 /* initialise counter */
 wait_fifo:
 	ldr 	r5, [r1, #0]	/* read wp */
 	cmp 	r5, #0			/* abort if wp == 0 */
@@ -59,6 +61,7 @@ wait_fifo:
 
 	ldr	r5, [r4]	/* "*target_address++ = *rp++" */
 	str	r5, [r3]
+        adds    r7, r5
 	adds	r4, #4
 	adds	r3, #4
 
@@ -72,4 +75,5 @@ no_wrap:
 	cmp     r0, #0
 	bne     wait_fifo		/* loop if not done */
 exit:
+        mov     r0, r7 /* store sum in r0 for result */
 	bkpt    #0
diff --git a/src/flash/nor/nrf51.c b/src/flash/nor/nrf51.c
index b20e26e..f8544cc 100644
--- a/src/flash/nor/nrf51.c
+++ b/src/flash/nor/nrf51.c
@@ -604,27 +604,32 @@ static int nrf51_erase_page(struct nrf51_info *chip, struct flash_sector *sector
 
 static const uint16_t nrf51_flash_write_code[] = {
 	/* See contrib/loaders/flash/cortex-m0.S */
-/*<wait_fifo>:*/
+	0x2700,      	/* movs	r7, #0 */
+/* <wait_fifo>: */
 	0x680d,      	/* ldr	r5, [r1, #0] */
 	0x2d00,      	/* cmp	r5, #0 */
-	0xd00e,      	/* beq.n	24 <exit> */
+	0xd00f,      	/* beq.n	28 <exit> */
 	0x684c,      	/* ldr	r4, [r1, #4] */
 	0x42ac,      	/* cmp	r4, r5 */
-	0xd0f9,      	/* beq.n	0 <wait_fifo> */
+	0xd0f9,      	/* beq.n	2 <wait_fifo> */
 	0x6825,      	/* ldr	r5, [r4, #0] */
 	0x601d,      	/* str	r5, [r3, #0] */
+	0x197f,      	/* adds	r7, r7, r5 */
 	0x3404,      	/* adds	r4, #4 */
 	0x3304,      	/* adds	r3, #4 */
 	0x4294,      	/* cmp	r4, r2 */
-	0xd301,      	/* bcc.n	1c <no_wrap> */
+	0xd301,      	/* bcc.n	20 <no_wrap> */
 	0x460c,      	/* mov	r4, r1 */
 	0x3408,      	/* adds	r4, #8 */
-/*<no_wrap>:*/
+
+/* <no_wrap>: */
 	0x604c,      	/* str	r4, [r1, #4] */
 	0x3804,      	/* subs	r0, #4 */
 	0x2800,      	/* cmp	r0, #0 */
-	0xd1ed,      	/* bne.n	0 <wait_fifo> */
-/*<exit>:*/
+	0xd1ec,      	/* bne.n	2 <wait_fifo> */
+
+/* <exit>: */
+	0x4638,      	/* mov	r0, r7 */
 	0xbe00,      	/* bkpt	0x0000 */
 };
 
@@ -692,6 +697,8 @@ static int nrf51_ll_flash_write(struct nrf51_info *chip, uint32_t offset, const
 	target_free_working_area(target, source);
 	target_free_working_area(target, write_algorithm);
 
+	LOG_ERROR("Sum over all bytes 0x%"PRIx32, buf_get_u32(reg_params[0].value, 0, 32));
+
 	destroy_reg_param(&reg_params[0]);
 	destroy_reg_param(&reg_params[1]);
 	destroy_reg_param(&reg_params[2]);
-- 
2.0.0

